function [dashTuning, dashScore, sig] = calculateDashModulation(speed,S)

% calculateDashModulation calculates the dash modulation, dash score and
% significance of the dash score for an individual cell
%
% dashTuning = calculateDashModulation(headAngle, S)
% calculates the head direction tuning map. Inputs:
%   speed: animal's speed in mm/s.
%   S: Vector of spike amplitudes
%
% [HDTuning,MVL] = placeCellAnalysis2022(...) also returns the mean vector
% length of the tuning curve
%
% [map, info, pSpatial] = placeCellAnalysis2022(...) also returns the
% p value that the cell has significant head direction tuning
%
% Marissa Applegate, Aronov Lab 2022

FR = 20;
hopThresh = 150;
nShuff = 500;
timeAroundHop = 2;

% find speedCrossings
low = speed<hopThresh;
high = speed>hopThresh;
crossing = low(1:end-1) & high(2:end);
range = 1:length(crossing);
frameCross = range(crossing==1);
%
for i = 1:length(frameCross)-1
      j = i+1;
      if ~isnan(frameCross(j))
  
        while j<=length(frameCross) && (frameCross(j)-frameCross(i))< FR %eliminate spurious dashes under 1s
            frameCross(j) = nan;
            j = j+1;
        end
      end 
end
frameCross(isnan(frameCross))= [];

numFrames = FR*timeAroundHop*2 + 1;
nFHalf = FR*timeAroundHop;

% calculate the observed tuning curve ans score
[dashTuning, dashScore] = calculateTuningAndScore(frameCross,S,numFrames,nFHalf);

dashScoreShuff = zeros(nShuff,1);
for ii = 1:nShuff
    offset = randi(size(S),1);
    %circularly permute
    if offset>1
    S = [S(offset-1:end) S(1:offset)];
    end

[~, dashScoreShuff(ii)] = calculateTuningAndScore(frameCross,S,numFrames,nFHalf);
end

sig = sum(dashScore<dashScoreShuff)/nShuff;
end


%helper function to calculate dash score and tuning curve
function [dashTuning, dashScore] = calculateTuningAndScore(frameCross,S,numFrames,nFHalf)

allTimes = nan(numFrames,length(frameCross)); % crosses by time by neurons

for i = 1:length(frameCross)
    %%
    offset1 = 1;
    offset2 = numFrames;
    f1 = frameCross(i)-nFHalf;
    f2 = frameCross(i)+nFHalf;
    if f1<=0
        offset1 = abs(f1) + 2;
        f1 = 1;
    elseif f2>= size(S,2)
        offset2 = numFrames - (f2-size(S,2));
        f2 = size(S,2);
    end      
    allTimes(offset1:offset2,i) = S(f1:f2);
end
%%

dashTuning =  nanmean(allTimes,2)' * 20; %adjust for the frame rate
meanFR = movmean(dashTuning,5);
baseline = min(meanFR(nFHalf:nFHalf + 41));
peak = max(meanFR(nFHalf:nFHalf + 41));

dashScore = (peak-baseline)./peak;
end



