function [HDTuning,MVL,sig] = calculateHDTuning(headAngle, S)
% calculateHDTuning calculate the head direction tuning, MVL score and
% significance for an individual cell
%
% HDTuning = calculateHDTuning(headAngle, S)
% calculates the head direction tuning map. Inputs:
%   headAngle: animal's head direction in radians relative.
%   S: Vector of spike amplitudes
%
% [HDTuning,MVL] = calculateHDTuning(...) also returns the mean vector
% length of the tuning curve
%
% [HDTuning,MVL, sig] = calculateHDTuning(...) also returns the
% p value that the cell has significant head direction tuning
%
% Marissa Applegate, Aronov Lab 2022

nbins = 40;
nShuff = 500;
fps = 20;

% get baseline occupancy
[baseline,range,bin] = histcounts(headAngle,linspace(-pi,pi,nbins+1));
%%
HDTuning = zeros(size(baseline));
for i = 1:nbins
    HDTuning(i) = sum(S(bin == i));
end

HDTuning = HDTuning ./ baseline * fps;

% smooth
w = gausswin(4, 2);
HDTuning = repmat(HDTuning,1,3);
HDTuning = filtfilt(w,1,HDTuning);
HDTuning = HDTuning(nbins+1:nbins*2);
%%
rangeMid = (range(1:end-1) + range(2:end))/2;

%% find the peak threshold
MVLAll= zeros(1,nShuff);

% calculate the random offset
for i = 1:nShuff
    offset = randi(size(S),1);

    %circularly permute
    if offset>1
    S = [S(offset-1:end) S(1:offset)];
    end
    
    angleFiringShuff = zeros(size(baseline));
    for b = 1:nbins
        angleFiringShuff(b) = sum(S(bin == b));
    end

    angleFiringShuff = angleFiringShuff ./ baseline;
% 
    angleFiringShuff = repmat(angleFiringShuff,1,3);
    angleFiringShuff = filtfilt(w,1,angleFiringShuff);
    angleFiringShuff = angleFiringShuff(nbins+1:nbins*2);

    MVLAll(i) = meanVectorLength(rangeMid,angleFiringShuff);
end

MVL = meanVectorLength(rangeMid,HDTuning);
sig = sum(MVL<MVLAll)/nShuff;

end

function MVL = meanVectorLength(th,rate)
nbins = length(rate);
resultant_vector = pi/(nbins*sin(pi/nbins)) * sum(rate.*exp(1i*th)) / sum(rate);
MVL = abs(resultant_vector);
end