function [map, info, pSpatial, timeShiftedInfo, threshMap, BS, pBS] = calculateSpatialTuning(xx, yy, S)
% calculateSpatialTuning Create place maps and calculate spatial info
%
% map = calculateSpatialTuning(xx, yy, S)
% calculates the place map. Inputs:
%   XX and YY: animal's position *(in mm)
%   S: Vector of spike amplitudes
%
% [map, info] = calculateSpatialTuning(...) also returns the normalized 
% spatial information accoridng to Skaggs et al. 1993, as a z-score.
%
% [map, info, pSpatial] = calculateSpatialTuning(...) also returns the
% p value that the cell is spatially significant
%
% [map, info, pSpatial, timeShiftedInfo] = 
% calculateSpatialTuning(...) also returns the tuning curve of spatial
% information shifted by time
%
% [map, info, pSpatial, timeShiftedInfo, threshMap] 
% = calculateSpatialTuning(...) also returns the map of the map of the 
% location of each field
%
% [map, info, pSpatial, timeShiftedInfo, threshMap, BS] =
% calculateSpatialTuning(...) also returns the border score
%
% [map, info, pSpatial, timeShiftedInfo, timeMaxShift, BS, pBS] = 
% calculateSpatialTuning(...) also returns the significance of the border
% score
%
% Adapted from Payne et al. 2021
%
% Marissa Applegate, Aronov Lab 2022

fps = 20;
lims = [20 900];
nbins = 40;
nhamming = 13;
nshuffle = 500;

%binarize ispk
ind = 1:length(S);
ispk = ind(S>0);
S_sp = S(ispk);

% Preallocate info vector
infoShuff = zeros(1,nshuffle);
BSShuff = zeros(1,nshuffle);

% Get bin edges
bin_edges = linspace(lims(1), lims(2), nbins+1);

% Get hamming filter
w = hamming(nhamming)*hamming(nhamming)';
w = w/sum(w(:));

% Calculate position counts (doesn't change with shuffling)
pos_count = histcounts2(xx, yy, bin_edges, bin_edges)'; % Transpose because it places the x categories in rows not columns
pos_time = pos_count/fps;
pos_time_smooth = nanconv(pos_time, w,'nanout', 'edge'); % should have no effect since there shouldn't be any nans, but important when making gaze maps

% Calculat the spatial map and observed spatial information
xx_sp = xx(ispk);
yy_sp = yy(ispk);
[map, infoReal] = calculateMap(xx_sp, yy_sp, S_sp ,w, pos_time_smooth,bin_edges);

% Calculate the number of fields
threshMap = findFields(map);

% Calculate the border score
BS = calculateBorderScore(threshMap, map);

tShift = -2*fps:2*fps;

timeShiftedInfo = zeros(size(tShift));
% shift the spatial information in time
for r = 0:length(tShift)-1
    tCurr = r - 2*fps;

    % permute spikes
    ispk = mod(ispk +tCurr-1, length(xx)) + 1;   
    
    % Get xy locations during spike times
    xx_sp = xx(ispk);
    yy_sp = yy(ispk);

    [~, timeShiftedInfo(r+1)] = calculateMap(xx_sp, yy_sp, S_sp, w, pos_time_smooth,bin_edges);

end

% get shuffled info
for i_shuffle = 1:max(1,nshuffle)
    % permute spikes
    r = ceil(rand*length(xx));
    ispk = mod(ispk + r - 1, length(xx)) + 1;   
    
    % Get xy locations during spike times
    xx_sp = xx(ispk);
    yy_sp = yy(ispk);

    [mShuff, infoShuff(i_shuffle)] = calculateMap(xx_sp, yy_sp, S_sp, w,pos_time_smooth,bin_edges);

    % Calculate the number of fields
    threshMapShuff = findFields(mShuff);
    
    % Calculate the border score
    BSShuff(i_shuffle) = calculateBorderScore(threshMapShuff, mShuff);

end

% get normalized spatial information
info = (infoReal - mean(infoShuff))/std(infoShuff);
pSpatial = sum(infoReal<infoShuff)/nshuffle;

pBS = sum(BS<BSShuff)/nshuffle;
end

% helper function that calculates the map and information
function [map, info] = calculateMap(xx_sp, yy_sp, S_sp, w,pos_time_smooth,bin_edges)
min_occup = .1;
nbins = 40;

% Get heat map
    [~,~,~,binX,binY] = histcounts2(xx_sp,yy_sp, bin_edges, bin_edges);
    binX(binX==0) = nan; binY(binY==0) = nan;
    subs = binX + (binY-1)*nbins; 
    mask = isnan(subs); 
    subs = subs(~mask);
    data = S_sp(~mask)';
    matrix=accumarray(subs',data, [nbins.^2,1]); sp_count = reshape(matrix,nbins,nbins)';

    % Filter while dealing with NaNs (file exchange function nanconv)
    sp_count_smooth = nanconv(sp_count, w,'nanout', 'edge');
    map = sp_count_smooth./pos_time_smooth;
    
    % Require a minimum time spent in this region
    map(pos_time_smooth<=min_occup) = NaN;
    pos_time_smooth(pos_time_smooth<=min_occup) = NaN;
    
    % Spatial information per spike (note: this is bits/spike, not bits per
    % second like in Skaggs et al. 1993).
    f = ~isnan(map);
    h = sp_count_smooth(f);
    t = pos_time_smooth(f);
    lambdai = h./t;             
    p_i = t/sum(t);
    lambda = sum(p_i.*lambdai); % mean rate
    f = lambdai>0;
    info = sum(p_i(f).*(lambdai(f)/lambda).*log2(lambdai(f)/lambda));
end

% helper function that calculates the number and location of fields
function  threshMap = findFields(map)
%%
minPixels = 40;
thresh = 0.5;
% subtract the minimum firing rate
map = (map - prctile(map(:),5)) ./ (prctile(map(:),95)-prctile(map(:),5));

% calculate size of regions and only include those larger than 200 cm^2
threshMap1 = map>thresh;
CC = bwconncomp(threshMap1, 4);
threshMap = zeros(size(map));
jj = 1;
for ii = 1:numel(CC.PixelIdxList)
    numPix = size(CC.PixelIdxList{ii},1);
    if numPix >= minPixels
        threshMap(CC.PixelIdxList{ii})= jj;
        jj = jj +1;
    end

end
end

% helper function that calculates the border score
function BS = calculateBorderScore(threshMap, map)
nbins = size(map,1);
threshMap = threshMap>0;
% have a map of distance to nearest wall
w1 = repmat( linspace(0,1,nbins),nbins,1);
w2 = w1'; w3 = 1-w1; w4 = 1-w2;
minDist = cat(3,w1,w2,w3,w4);
minDist = min(minDist,[],3);% normalize FR by its sum
Dm = mean(minDist(threshMap) .* map(threshMap));

% find maximum overlap with the walls
w1 = sum(threshMap(:,1)); w2 = sum(threshMap(:,end)); 
w3 = sum(threshMap(1,:)); w4 = sum(threshMap(end,:));

Cm = max([w1 w2 w3 w4])./ nbins;
%
BS = (Cm - Dm) / (Cm + Dm);
end