function [xcorr_val, sig] = calculateStability(xx, yy, S)
% calculateStability Create place maps and calculate spatial info
%
% xcorr_val = calculateStability(xx, yy, S)
% calculates average xcorrelation of the place map generated from random
% halves of the session.
%   XX and YY: animal's position *(in mm)
%   S: Vector of spike amplitudes
%
% [xcorr_val, sig] = calculateSpatialTuning(...) also returns the p-value
% of the stability score.
%
% Adapted from Payne et al. 2021
%
% Marissa Applegate, Aronov Lab 2022

fps = 20;
nbins = 40;
lims = [20 900];
nhamming = 13;

nshuffle = 500;
seg_duration = 300; 
nreps = 10;

% Preallocate xcorr vector
xcorr_all_obs = NaN(nreps,1);
xcorr_all_shuff = NaN(nreps, nshuffle);

% Get bin edges
bin_edges = linspace(lims(1), lims(2), nbins+1);

% Get hamming filter
w = hamming(nhamming)*hamming(nhamming)';
w = w/sum(w(:));

% get ispk and S_sp
ind = 1:length(S);
ispk = ind(S>0);
S_sp = S(ispk);

for jj = 1:nreps
    
    % Divide the data in two and calculate xcorr of place maps. Repeat this 10 x
    nt_per_seg = fps*seg_duration;
    segs = ceil((1:length(xx))/nt_per_seg);
    nsegs = max(segs);
    curr_seg_inds = randperm(nsegs, floor(nsegs/2));
    curr_mask = ismember(segs, curr_seg_inds);
    xx1 = xx;
    yy1 = yy;
    xx1(curr_mask) = NaN;
    yy1(curr_mask) = NaN;
    xx2 = xx;
    yy2 = yy;
    xx2(~curr_mask) = NaN;
    yy2(~curr_mask) = NaN;
    
    % Get time spent in each bin - doesn't depend on spikes so just do once
    pos_time_smooth1 = get_pos_time(xx1,yy1,bin_edges,fps,w);
    pos_time_smooth2 = get_pos_time(xx2,yy2,bin_edges,fps,w);
   
    % calculate observed correlation
    xcorr_all_obs(jj) =  calculatexCorr(xx1,xx2,yy1,yy2,ispk,S_sp,bin_edges,w,pos_time_smooth1,pos_time_smooth2);
    
    % calculate shuffled correlation
    for ii = 1:nshuffle
            r = ceil(rand*length(xx));
            ispk_curr = mod(ispk + r - 1, length(xx)) + 1;
            xcorr_all_shuff(jj,ii) =  calculatexCorr(xx1,xx2,yy1,yy2,...
                ispk_curr,S_sp,bin_edges,w,pos_time_smooth1,pos_time_smooth2);
    end

end
xcorr_val = nanmean(xcorr_all_obs,1);
xcorr_shuff = nanmean(xcorr_all_shuff,1);

sig = sum(xcorr_val<xcorr_shuff)/nshuffle;

end


function xc = calculatexCorr(xx1,xx2,yy1,yy2,ispk_curr,S_sp,bin_edges,w,pos_time_smooth1,pos_time_smooth2)
min_occup = .1;

% Get xy locations during spike times
spk_count_smooth1 = get_spk_count(xx1,yy1,ispk_curr,S_sp, bin_edges, w);
spk_count_smooth2 = get_spk_count(xx2,yy2,ispk_curr,S_sp, bin_edges, w);

% Get map: ratio of spk count/time occupied
map1 = spk_count_smooth1./pos_time_smooth1;
map2 = spk_count_smooth2./pos_time_smooth2;

% Require a minimum time spent in this region
map1(pos_time_smooth1<=min_occup) = NaN;
map2(pos_time_smooth2<=min_occup) = NaN;        

% Get the bins without NaN
mask = isnan(map1) | isnan(map2);
map1 = map1(~mask);
map2 = map2(~mask);

% Get 2D xcorr
n = numel(map1);
xc = (n*sum(map1.*map2) - sum(map1)*sum(map2))/(sqrt(n*sum(map1.^2)-sum(map1).^2).*sqrt(n*sum(map2.^2)-sum(map2).^2));        
if all(~map1) || all(~map2)
     xc = 0;
end
        


end


% Calculate position counts (doesn't change with shuffling)
function pos_time_smooth = get_pos_time(xx,yy,bin_edges,fps,w)
pos_count = histcounts2(xx, yy, bin_edges, bin_edges)'; % It places the x categories in rows not columns - switch
pos_time = pos_count/fps;
pos_time_smooth = nanconv(pos_time, w,'nanout', 'edge'); % nanout  should have no effect since there shouldn't be any nans, but important when making gaze maps
end

function spk_count_smooth = get_spk_count(xx,yy,ispk, S_sp,bin_edges, w)
  xx_sp = xx(ispk);
    yy_sp = yy(ispk);
nbins = length(bin_edges)-1;
    % Get heat map
    [~,~,~,binX,binY] = histcounts2(xx_sp,yy_sp, bin_edges, bin_edges);
    binX(binX==0) = nan; binY(binY==0) = nan;
    subs = binX + (binY-1)*nbins; mask = isnan(subs); subs = subs(~mask);
    data = S_sp(~mask)';
    matrix=accumarray(subs',data, [nbins.^2,1]); sp_count = reshape(matrix,nbins,nbins)';

    % Filter while dealing with NaNs (file exchange function nanconv)
    spk_count_smooth = nanconv(sp_count, w,'nanout', 'edge');
  
end